#include <iostream>
#include <algorithm>
#include <string>
#include <vector>
#include <set>
#include <map>
#include <queue>
#include <bitset>
#include <sstream>
#include <cstdio>
#include <cstring>
#include <cmath>
#include <climits>
#include <cctype>
#include <cstdlib>
using namespace std;

#define FOR(i,a,b) for(int i=(a);i<(b);i++)
#define ROF(i,a,b) for(int i=(b)-1;i>=(a);i--)

typedef long long LL;
typedef LL ll;
typedef pair<int, int> pii;
const int INF = 1000000000;
const LL LINF = 1000000000000000000LL;
const double PI = 3.1415926535897932384626433832795;

struct Vector2D {
  double x, y;

  Vector2D(double x = 0.0, double y = 0.0) : x(x), y(y) {}
};

Vector2D operator+(const Vector2D& u, const Vector2D& v) {
  return Vector2D(u.x + v.x, u.y + v.y);
}

Vector2D operator-(const Vector2D& u, const Vector2D& v) {
  return Vector2D(u.x - v.x, u.y - v.y);
}

double dot(const Vector2D& u, const Vector2D& v) {
  return u.x * v.x + u.y * v.y;
}

double cross(const Vector2D& u, const Vector2D& v) {
  return u.x * v.y - u.y * v.x;
}

Vector2D operator*(double k, const Vector2D& u) {
  return Vector2D(k * u.x, k * u.y);
}
Vector2D operator*(const Vector2D& u, double k) {
  return Vector2D(k * u.x, k * u.y);
}
typedef Vector2D Point2D;

struct Vector {
  double x, y, z;

  Vector(double x = 0.0, double y = 0.0, double z = 0.0) : x(x), y(y), z(z) {}
};

Vector operator+(const Vector& u, const Vector& v) {
  return Vector(u.x + v.x, u.y + v.y, u.z + v.z);
}

Vector operator-(const Vector& u, const Vector& v) {
  return Vector(u.x - v.x, u.y - v.y, u.z - v.z);
}

double dot(const Vector& u, const Vector& v) {
  return u.x * v.x + u.y * v.y + u.z * v.z;
}
Vector cross(Vector const& a, Vector const& b) {
  return Vector(a.y * b.z - a.z * b.y,
                a.z * b.x - a.x * b.z,
                a.x * b.y - a.y * b.x);
}
double length(Vector const& a) {
  return sqrt(dot(a, a));
}

Vector operator - (Vector const& a) {
  return Vector(-a.x, -a.y, -a.z);
}

Vector operator*(double k, const Vector& u) {
  return Vector(k * u.x, k * u.y, k * u.z);
}
Vector operator*(const Vector& u, double k) {
  return Vector(k * u.x, k * u.y, k * u.z);
}

typedef Vector Point;

struct Plane {
  Vector normal;
  double dist;
  Plane() {}
  Plane(Vector const& v, double d) : normal(v), dist(d) {
    double x = 1 / length(normal);
    normal = normal * x;
    dist *= x;
  }
};
vector<Plane> hull;
vector<Point> points;
bool v3equal(Plane const& a, Plane const& b) {
  return (fabs(a.normal.x - b.normal.x) < 1e-8) &&
         (fabs(a.normal.y - b.normal.y) < 1e-8) &&
         (fabs(a.normal.z - b.normal.z) < 1e-8);
}
bool v3less(Plane const& a, Plane const& b) {
  if (fabs(a.normal.x - b.normal.x) > 1e-8) return a.normal.x < b.normal.x;
  if (fabs(a.normal.y - b.normal.y) > 1e-8) return a.normal.y < b.normal.y;
  if (fabs(a.normal.z - b.normal.z) > 1e-8) return a.normal.z < b.normal.z;
  return a.dist < b.dist;
}
void HullAdd(Vector n, double d) {
  for (int i = 0; i < points.size(); i++) {
    if (dot(points[i], n) + d > 1e-8) {
      return;
    }
  }
  hull.push_back(Plane(n, d));
}

void BuildHull() {
  for (int i = 0; i < points.size(); i++) {
    for (int j = i + 1; j < points.size(); j++) {
      int k = 0;
      Point a = points[i];
      Point b = points[j];
      while (k == i || k == j) k++;
      Point left = points[k], right = points[k];
      Vector nleft = cross(left - a, b - a), nright = cross(right - b, a - b);
      double dleft = dot(nleft, a), dright = dot(nright, b);
      for (k++; k < points.size(); k++) {
        if (k == i || k == j) continue;
        if (dot(points[k], nleft) > dleft) {
          left = points[k];
          nleft = cross(left - a, b - a);
          dleft = dot(nleft, a);
        }
        if (dot(points[k], nright) > dright) {
          right = points[k];
          nright = cross(right - b, a - b);
          dright = dot(nright, b);
        }
      }
      HullAdd(nleft, -dleft);
      HullAdd(nright, -dright);
    }
  }
}

double volume = 0;
Point sum(0, 0, 0);
Point p0;
Vector normal;
Point middle(0, 0, 0);
bool p3less(Point const& a, Point const& b) {
  if (fabs(a.x - b.x) > 1e-8) return a.x < b.x;
  if (fabs(a.y - b.y) > 1e-8) return a.y < b.y;
  return a.z < b.z;
}
bool p3less2(Point const& a, Point const& b) {
  return dot(cross(a - p0, b - p0), normal) < 0;
}
double SumPlane(Plane const& p) {
  vector<Point> pt;
  for (int i = 0; i < points.size(); i++) {
    if (fabs(dot(points[i], p.normal) + p.dist) < 1e-8) {
      pt.push_back(points[i]);
    }
  }
  sort(pt.begin(), pt.end(), p3less);
  p0 = pt[0];
  normal = p.normal;
  sort(pt.begin() + 1, pt.end(), p3less2);
  double ov = volume;
  for (int i = 2; i < pt.size(); i++) {
    Point a = pt[0];
    Point b = pt[i - 1];
    Point c = pt[i];
    Point d = middle;
    Point m = (a + b + c + d) * 0.25;
    double h = fabs(dot(middle, p.normal) + p.dist);
    double area = length(cross(b - a, c - a)) * 0.5;
    double vol = area * h / 3;
    sum = sum + m * vol;
    volume += vol;
  }
  return volume - ov;
}

int main() {
  double Dist[2] = {1e+9, 1e+9};
  for (int F = 0; F < 2; F++) {
    int n;
    cin >> n;
    points.clear();
    hull.clear();
    FOR(i, 0, n) {
      double x, y, z;
      cin >> x >> y >> z;
      points.push_back(Vector(x, y, z));
    }
    BuildHull();
    sort(hull.begin(), hull.end(), v3less);
    hull.resize(unique(hull.begin(), hull.end(), v3equal) - hull.begin());
    volume = 0;
    sum = middle = Vector(0, 0, 0);
    for (int i = 0; i < n; i++) {
      middle = middle + points[i];
    }
    middle = middle * (1.0 / n);
    for (int i = 0; i < hull.size(); i++) {
      SumPlane(hull[i]);
    }
    sum = sum * (1.0 / volume);
    for (int i = 0; i < hull.size(); i++) {
      Dist[F] = min(Dist[F], fabs(dot(sum, hull[i].normal) + hull[i].dist));
    }
  }
  cout.precision(9);
//  cout << Dist[0] << " " << Dist[1];
  cout << fixed << Dist[0] + Dist[1];
  return 0;
}
