#include <iostream>
#include <cassert>
#include <cstdio>
#include <vector>
#include <algorithm>
#include <complex>
using namespace std;

typedef complex<double> point;
typedef vector<point> polygon;

template <class T> int size(const T &x) { return x.size(); }

const double EPS = 1e-9;

double dot(point a, point b) {
    return real(conj(a) * b);
}
double cross(point a, point b) {
    return imag(conj(a) * b);
}

double ccw(point a, point b, point c) {
    return cross(b - a, c - b);
}

double polygon_area_signed(polygon p) {
    double area = 0;
    int cnt = p.size();
    for (int i = 1; i + 1 < cnt; i++) {
        area += cross(p[i] - p[0], p[i+1] - p[0]);
    }
    return area/2.0;
}
double polygon_area(polygon p) {
    return abs(polygon_area_signed(p));
}

#define MAXN 1000
point hull[MAXN];
bool cmp(const point &a, const point &b) {
    return abs(real(a) - real(b)) > EPS ? real(a) < real(b) : imag(a) < imag(b);
}

int convex_hull(polygon p) {
    int n = size(p), l = 0;
    sort(p.begin(), p.end(), cmp);
    for (int i = 0; i < n; i++) {
        if (i > 0 && p[i] == p[i-1]) continue;
        while (l >= 2 && ccw(hull[l-2], hull[l-1], p[i]) >= 0) l--;
        hull[l++] = p[i];
    }
    int r = l;
    for (int i = n - 2; i >= 0; i--) {
        if (p[i] == p[i+1]) continue;
        while (r - l >= 1 && ccw(hull[r-2], hull[r-1], p[i]) >= 0) r--;
        hull[r++] = p[i];
    }
    return l == 1 ? 1 : r - 1;
}

struct Point {
    double x, y, z;
    Point() {}
    Point(double x, double y, double z) : x(x), y(y), z(z) {}

    Point operator *(double k) const {
        return Point(k * x, k * y, k * z);
    }
    Point operator /(double k) const {
        return Point(x / k, y / k, z / k);
    }
    double operator %(const Point &p) const {
        return x * p.x + y * p.y + z * p.z;
    }
    Point operator *(const Point &p) const {
        return Point(
                    y * p.z - z * p.y,
                    z * p.x - x * p.z,
                    x * p.y - y * p.x
                );
    }
    Point operator +(const Point &p) const {
        return Point(
                x + p.x,
                y + p.y,
                z + p.z
                );
    }
    Point operator -(const Point &p) const {
        return Point(
                x - p.x,
                y - p.y,
                z - p.z
                );
    }
};

int getIntersection(
        Point &a,
        Point &b,

        Point &c, 
        Point &d,
        Point &e,

        Point &o) {

    Point v = b - a;
    double V1 = (c - a) * (d - a) % (e - a);
    double V2 = (d - b) * (c - b) % (e - b);
    double V = V1 + V2;
    assert(abs(V) > EPS);
    v = v / V;
    /*
    if (abs(V - 0.0) < EPS) {
        if (a.isOnPlane(c, d, e)) return 2;
        return 0;
    }
    */
    v = v * V1;
    o = a + v;
    return 1;
}

vector<point> pts[1000];
//polygon chull[110];

int main() {

    int n, z0, z1;
    scanf("%d %d %d\n", &n, &z0, &z1);

    for (int i = 0; i < n; i++) {
        double x, y;
        int z;
        scanf("%lf %lf %d\n", &x, &y, &z);

        //assert(z0 <= z && z <= z1);
        pts[z + 200].push_back(point(x, y));
    }

    /*
    assert(size(pts[z0]) > 0);
    assert(size(pts[z1]) > 0);
    */

    //vector<int> nonempty;
    for (int z = z0; z <= z1; z++) {
        // todo: edge cases

        //nonempty.push_back(z);

        polygon p(pts[z + 200]);

        for (int zl = -200; zl <= z - 1; zl++) {
            for (int zr = z + 1; zr <= 200; zr++) {
                for (int a = 0; a < size(pts[zl + 200]); a++) {
                    for (int b = 0; b < size(pts[zr + 200]); b++) {
                        point x = pts[zl + 200][a],
                              y = pts[zr + 200][b];

                        Point A(0, 0, z),
                              B(1, 0, z),
                              C(0, 1, z);

                        Point X = Point(real(x), imag(x), zl),
                              Y = Point(real(y), imag(y), zr);

                        Point res;
                        assert(getIntersection(X, Y, A, B, C, res) == 1);
                        p.push_back(point(res.x, res.y));
                    }
                }

            }
        }

        int cnt = convex_hull(p);
        polygon q;
        for (int i = 0; i < cnt; i++) {
            q.push_back(hull[i]);
        }

        //chull[z] = p;

        printf("%0.10lf\n", polygon_area(q));
    }

    /*
    for (int i = 1; i < nonempty.size(); i++) {
        int zl = nonempty[i-1],
            zr = nonempty[i];


        for (int z = zl + 1; z <= zr - 1; z++) {

            polygon p;
            for (int a = 0; a < chull[zl].size(); a++) {
                for (int b = 0; b < chull[zr].size(); b++) {
                    point x = chull[zl][a],
                          y = chull[zr][b];

                    Point A(0, 0, z),
                          B(1, 0, z),
                          C(0, 1, z);

                    Point X = Point(real(x), imag(x), zl),
                          Y = Point(real(y), imag(y), zr);

                    Point res;
                    assert(getIntersection(X, Y, A, B, C, res) == 1);
                    p.push_back(point(res.x, res.y));
                }
            }

            int cnt = convex_hull(p);
            chull[z].clear();
            for (int j = 0; j < cnt; j++) {
                chull[z].push_back(hull[j]);
            }
        }
    }
    */

    /*
    for (int z = z0; z <= z1; z++) {
        printf("%0.10lf\n", polygon_area(chull[z]));
    }
    */
}

